/*
 * Copyright (c) 2011 J. Lewis Muir <jlmuir@imca.aps.anl.gov>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jaccordion;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

/**
 * I am a panel that can be opened or closed by clicking on my heading.
 */
public class JClosablePanel extends JPanel {
  private static final long serialVersionUID = 1L;

  private static final Icon OPEN_ICON = createOpenIcon();
  private static final Icon CLOSED_ICON = createClosedIcon();

  private final JLabel heading;
  private final Component body;
  private boolean open;
  private final ArrayList<WeakReference<OpenedListener>> openedListeners;

  public JClosablePanel(String title, Component body) {
    super(new BorderLayout());
    setName(title);
    heading = createHeading(title);
    heading.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        toggleOpen();
      }
    });
    add(heading, BorderLayout.NORTH);
    this.body = body;
    add(body, BorderLayout.CENTER);
    body.setVisible(false);
    open = false;
    openedListeners = new ArrayList<WeakReference<OpenedListener>>();
    setBorder(createBorder());
  }

  public String getTitle() {
    return heading.getText();
  }

  public Component getBody() {
    return body;
  }

  public boolean isOpen() {
    return open;
  }

  public void setOpen(boolean newOpen) {
    if (open == newOpen) return;
    open = newOpen;
    heading.setIcon(newOpen ? OPEN_ICON : CLOSED_ICON);
    body.setVisible(newOpen);
    if (newOpen) fireOpened();
  }

  public void makeOpen() {
    setOpen(true);
  }

  public void makeClosed() {
    setOpen(false);
  }

  public void toggleOpen() {
    setOpen(!open);
  }

  public Color getHeaderBackground() {
    return heading.getBackground();
  }

  public void setHeaderBackground(Color c) {
    heading.setBackground(c);
  }

  public Color getHeaderForeground() {
    return heading.getForeground();
  }

  public void setHeaderForeground(Color c) {
    heading.setForeground(c);
  }

  public Font getTitleFont() {
    return heading.getFont();
  }

  public void setTitleFont(Font f) {
    heading.setFont(f);
  }

  synchronized void addOpenedListener(OpenedListener listener) {
    if (findOpenedListener(listener) >= 0) return;
    openedListeners.add(new WeakReference<OpenedListener>(listener));
  }

  synchronized void removeOpenedListener(OpenedListener listener) {
    int index = findOpenedListener(listener);
    if (index >= 0) openedListeners.remove(index);
  }

  private synchronized int findOpenedListener(OpenedListener listener) {
    int i = 0;
    for (WeakReference<OpenedListener> each : openedListeners) {
      if (listener.equals(each.get())) return i;
      i++;
    }
    return -1;
  }

  private synchronized void fireOpened() {
    for (Iterator<WeakReference<OpenedListener>> i = openedListeners.iterator(); i.hasNext();) {
      OpenedListener listener = i.next().get();
      if (listener == null) {
        i.remove();
      } else {
        listener.opened(this);
      }
    }
  }

  private static Icon createOpenIcon() {
    return new ImageIcon(createOpenImage());
  }

  private static Icon createClosedIcon() {
    return new ImageIcon(createClosedImage());
  }

  private static BufferedImage createOpenImage() {
    BufferedImage image = createClosedImage();
    AffineTransform at = new AffineTransform();
    at.rotate(Math.PI / 2.0, image.getWidth() / 2.0, image.getHeight() / 2.0);
    AffineTransformOp atOp = new AffineTransformOp(at, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
    return atOp.filter(image, null);
  }

  private static BufferedImage createClosedImage() {
    BufferedImage image = new BufferedImage(7, 7, BufferedImage.TYPE_INT_ARGB);
    Graphics2D g = image.createGraphics();
    Polygon triangle = new Polygon();
    triangle.addPoint(2, 0);
    triangle.addPoint(5, 3);
    triangle.addPoint(2, 6);
    g.setColor(Color.BLACK);
    g.drawPolygon(triangle);
    g.fillPolygon(triangle);
    return image;
  }

  private static JLabel createHeading(String title) {
    JLabel result = new JLabel(title, CLOSED_ICON, SwingConstants.LEADING);
    int bwidth = result.getIconTextGap();
    if (bwidth < 4) bwidth = 4;
    result.setBorder(BorderFactory.createEmptyBorder(bwidth, bwidth, bwidth, bwidth));
    result.setOpaque(true);
    result.setBackground(darker(result.getBackground()));
    return result;
  }

  private static Border createBorder() {
    return BorderFactory.createLineBorder(getDefaultLabelBackground().darker(), 1);
  }

  private static Color getDefaultLabelBackground() {
    return (new JLabel()).getBackground();
  }

  private static Color darker(Color c) {
    float[] components = c.getRGBColorComponents(null);
    for (int i = 0; i < components.length; i++) {
      components[i] *= 0.96f;
      if (components[i] > 1.0f) components[i] = 1.0f;
    }
    return new Color(components[0], components[1], components[2]);
  }
}
