/*
 * Copyright (c) 2011 J. Lewis Muir <jlmuir@imca.aps.anl.gov>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jaccordion.demo;

import imcacat.jaccordion.JAccordion;
import imcacat.jaccordion.JClosablePanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

/**
 * I am a demonstration of <code>JAccordion</code>. In addition to the
 * <code>JAccordion</code> component, I provide a Tools menu for opening an
 * inspector for making run-time changes to my <code>JAccordion</code>
 * instance.
 */
public class JAccordionDemo {
  private static final Dimension FRAME_SIZE = new Dimension(200, 200);

  public static void main(String[] args) {
    setSpecialSystemProperties();

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        try {
          JAccordion accordion = createAccordion();
          JFrame frame = new JFrame("JAccordionDemo");
          frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
          frame.setJMenuBar(createMenuBar(createAccordionInspectorDialog(accordion, frame)));
          frame.getContentPane().add(accordion, BorderLayout.CENTER);
          frame.pack();
          Dimension size = frame.getPreferredSize();
          if (FRAME_SIZE.width > size.width) size.width = FRAME_SIZE.width;
          if (FRAME_SIZE.height > size.height) size.height = FRAME_SIZE.height;
          frame.setSize(size);
          frame.setLocationRelativeTo(null);
          frame.setVisible(true);
        } catch (Exception e) {
          throw new RuntimeException(e);
        }
      }
    });
  }

  private static void setSpecialSystemProperties() {
    System.setProperty("apple.laf.useScreenMenuBar", "true");
  }

  private static JAccordion createAccordion() {
    JAccordion result = new JAccordion();
    result.addSection("Characters", createCharactersPanel());
    result.addSection("Places", createPlacesPanel());
    result.addSection("Ships", createShipsPanel());
    return result;
  }

  private static JPanel createCharactersPanel() {
    return createWhateverPanel();
  }

  private static JPanel createPlacesPanel() {
    return createWhateverPanel();
  }

  private static JPanel createShipsPanel() {
    return createWhateverPanel();
  }

  private static JPanel createWhateverPanel() {
    JPanel result = new JPanel();
    result.add(new JLabel("Whatever"));
    return result;
  }

  private static JDialog createAccordionInspectorDialog(JAccordion accordion, JFrame owner) {
    JDialog result = new JDialog(owner, "JAccordionDemo Inspector");
    result.getContentPane().add(createAccordionInspector(accordion), BorderLayout.CENTER);
    result.pack();
    result.setResizable(false);
    return result;
  }

  private static JComponent createAccordionInspector(final JAccordion accordion) {
    JPanel result = new JPanel();
    GridBagLayout layout = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    result.setLayout(layout);

    JLabel atLabel = new JLabel("At");
    result.add(atLabel, c);

    ArrayList<SectionStandIn> sections = new ArrayList<SectionStandIn>();
    for (Iterator<JClosablePanel> i = accordion.getSections(); i.hasNext();) {
      sections.add(new SectionStandIn(i.next().getTitle()));
    }
    sections.add(SectionStandIn.BOTTOM);
    final JComboBox sectionsCombo = new JComboBox(sections.toArray());
    sectionsCombo.setPrototypeDisplayValue("MMMMMMMMMMMMMMM");
    c.fill = GridBagConstraints.HORIZONTAL;
    result.add(sectionsCombo, c);

    JLabel colonLabel = new JLabel(":");
    c.fill = GridBagConstraints.NONE;
    result.add(colonLabel, c);

    final JButton removeButton = new JButton("Remove");
    if (sections.size() <= 1) removeButton.setEnabled(false);
    c.fill = GridBagConstraints.HORIZONTAL;
    result.add(removeButton, c);

    JButton insertButton = new JButton("Insert");
    c.gridx = 3;
    c.gridy = 1;
    result.add(insertButton, c);

    final JTextField sectionTitleField = new JTextField("Creatures", 15);
    c.weightx = 1.0;
    c.gridx = GridBagConstraints.RELATIVE;
    result.add(sectionTitleField, c);

    JCheckBox maxOneOpenCheck =
      new JCheckBox("Only one section may be open at a time",
        accordion.isMaxOneSectionOpenEnabled());
    c.fill = GridBagConstraints.NONE;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.LINE_START;
    c.gridx = 0;
    c.gridy = 2;
    result.add(maxOneOpenCheck, c);

    sectionsCombo.addItemListener(new ItemListener() {
      @Override
      public void itemStateChanged(ItemEvent event) {
        if (event.getStateChange() == ItemEvent.DESELECTED) return;
        removeButton.setEnabled(!SectionStandIn.BOTTOM.equals(event.getItem()));
      }
    });

    removeButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent event) {
        int index = sectionsCombo.getSelectedIndex();
        if (index == -1) return;
        if (index == accordion.sectionCount()) return;
        accordion.removeSection(index);
        if (accordion.sectionCount() == 0) removeButton.setEnabled(false);
        sectionsCombo.removeItemAt(index);
      }
    });

    ActionListener insertActionListener = new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent event) {
        int index = sectionsCombo.getSelectedIndex();
        if (index == -1) index = accordion.sectionCount();
        String title = sectionTitleField.getText();
        accordion.insertSection(title, createWhateverPanel(), index);
        removeButton.setEnabled(true);
        sectionsCombo.insertItemAt(new SectionStandIn(title), index);
        sectionsCombo.setSelectedIndex(index);
      }
    };

    insertButton.addActionListener(insertActionListener);

    sectionTitleField.addActionListener(insertActionListener);

    maxOneOpenCheck.addItemListener(new ItemListener() {
      @Override
      public void itemStateChanged(ItemEvent event) {
        accordion.setMaxOneSectionOpenEnabled(event.getStateChange() == ItemEvent.SELECTED);
      }
    });

    result.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
    return result;
  }

  private static JMenuBar createMenuBar(final JDialog accordionInspector) {
    JMenuBar result = new JMenuBar();

    JMenu toolsMenu = new JMenu("Tools");
    toolsMenu.setMnemonic(KeyEvent.VK_T);
    Action openInspectorAction = new AbstractAction() {
      private static final long serialVersionUID = 1;

      public void actionPerformed(ActionEvent event) {
        if (accordionInspector.isVisible()) return;
        accordionInspector.setLocationRelativeTo(accordionInspector.getOwner());
        accordionInspector.setVisible(true);
      }
    };
    openInspectorAction.putValue(Action.NAME, "Show Inspector");
    JMenuItem inspectorMenuItem = new JMenuItem(openInspectorAction);
    inspectorMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,
      Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
    toolsMenu.add(inspectorMenuItem);
    result.add(toolsMenu);

    return result;
  }
}
